@@ -1,4 +1,5 @@
 import urllib.request, json
+import logging
 
 import jose.jwt
 from starlette.requests import Request
@@ -6,8 +7,12 @@ from starlette.responses import JSONResponse
 from starlette.types import ASGIApp, Receive, Scope, Send
 
 
+logger = logging.getLogger(__name__)
+
+
 def _get_keys(url_or_keys):
     if isinstance(url_or_keys, str) and url_or_keys.startswith('https://'):
+        logger.info("Getting jwk from {}...", url_or_keys)
         with urllib.request.urlopen(url_or_keys) as f:
             return json.loads(f.read().decode())
     else:
@@ -62,12 +67,17 @@ class AuthenticateMiddleware:
         The ``at_hash`` value in the token, if present, is ignored, because in order to
         verify it, the access token would be required, which we don't have access to.
         """
-        return jose.jwt.decode(
+        issuer = self._providers[provider]['issuer']
+        audience = self._providers[provider]['audience']
+        logger.debug("Trying to decode token for provider \"%s\", issuer \"%s\", audience \"%s\"...", provider, issuer, audience)
+        decoded = jose.jwt.decode(
             token, self._provider_keys(provider),
-            issuer=self._providers[provider]['issuer'],
-            audience=self._providers[provider]['audience'],
+            issuer=issuer,
+            audience=audience,
             options={'verify_at_hash': False}  # Ignore at_hash, if present
         )
+        logger.debug("Token decoded.")
+        return decoded
 
     def claims(self, token: str):
         """
@@ -80,9 +90,14 @@ class AuthenticateMiddleware:
                 return self._provider_claims(provider, token)
             except jose.exceptions.ExpiredSignatureError as e:
                 # if the token has expired, it is at least from this provider.
+                logger.debug("Token has expired.")
                 errors = str(e)
                 break
+            except jose.exceptions.JWTClaimsError as e:
+                logger.debug("Invalid claims")
+                errors[provider] = str(e)
             except jose.exceptions.JOSEError as e:  # the catch-all of Jose
+                logger.warning(e, exc_info=True)
                 errors[provider] = str(e)
         raise InvalidToken(errors)
 
@@ -101,9 +116,11 @@ class AuthenticateMiddleware:
                 response = JSONResponse({'message': e.errors}, status_code=401)
                 return await response(scope, receive, send)
         elif 'authorization' in request.headers:
+            logger.debug("No \"Bearer\" in authorization header")
             response = JSONResponse({'message': 'The "authorization" header must start with "Bearer "'}, status_code=400)
             return await response(scope, receive, send)
         else:
+            logger.debug("No authorization header")
             response = JSONResponse({'message': 'The request does not contain an "authorization" header'}, status_code=400)
             return await response(scope, receive, send)
 
